/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.misc.android;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;

import netacka3d.data.Util3d;
import netacka3d.misc.FileIOBackend;
import netacka3d.misc.Logger;
import android.content.Context;
import android.content.res.Resources;
import android.content.res.Resources.NotFoundException;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;

public class AndroidFileIOBackend extends FileIOBackend {

    private Context context;
    
    public AndroidFileIOBackend(Context context) {
        assert context != null;
        this.context = context;
    }

    @Override
    public InputStream openFile(String filename) throws FileNotFoundException {
        String resName = filename.substring(0, filename.indexOf('.'));
        Logger.i("opening raw resource " + resName);
        Resources res = context.getResources();
        int id = res.getIdentifier(resName, "raw", "netacka3d.ui.android");
        assert id != 0;
        try {
            return res.openRawResource(id);
        }
        catch (NotFoundException e) {
            throw new FileNotFoundException(e.toString());
        }
    }

    @Override
    public ByteBuffer loadImage(String filename, int[] dims)
            throws IOException {
        Logger.i("Loading image " + filename);
        long t = System.currentTimeMillis();
        
        Resources res = context.getResources();
        String resName = filename.substring(0, filename.indexOf('.'));
        int id = res.getIdentifier(resName, "drawable", "netacka3d.ui.android");
        Bitmap bmp = BitmapFactory.decodeResource(res, id);
        int w = bmp.getWidth(), h = bmp.getHeight();
        if (dims != null) {
            dims[0] = w;
            dims[1] = h;
        }
        
        int[] data = new int[w*h];
        bmp.getPixels(data, 0, w, 0, 0, w, h);
        ByteBuffer buf = Util3d.newPixelBuffer(data);
        Logger.i("Loaded in " + (System.currentTimeMillis() - t) + " msecs");
        return buf;
    }
}
