/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.ui.android;

import netacka3d.data.Direction;
import netacka3d.graphics.android.GameRenderer;
import netacka3d.logic.ClientController;
import android.content.Context;
import android.opengl.GLSurfaceView;
import android.util.Log;
import android.view.KeyEvent;
import android.view.MotionEvent;

/**
 * Trójwymiarowy widok planszy, do umieszczenia w GameWindow. 
 */
public class GameView extends GLSurfaceView {
    public static final int FPS = 10;
    
    private GameRenderer renderer;
//    private Controller controller;
    private ClientController controller;
    private final float sqrt3 = (float) Math.sqrt(3);
    private RenderThread renderThread;
    
    private int key_x = 0, key_y = 0, key_z = 0;
    
    public GameView(Context context, ClientController controller) {
        super(context);
        this.controller = controller;
        renderer = new GameRenderer(controller);
        setRenderer(renderer);
        setRenderMode(RENDERMODE_WHEN_DIRTY);
        renderThread = new RenderThread();
        renderThread.start();
        // bez tego nie działa klawiatura
        setFocusable(true);
        setFocusableInTouchMode(true);
    }    
    
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent msg) {
        Log.i("netacka", "keyDown "+keyCode);
        switch (keyCode) {
        case KeyEvent.KEYCODE_A: key_y =  1; break;
        case KeyEvent.KEYCODE_D: key_y = -1; break;
        case KeyEvent.KEYCODE_W: key_x =  1; break;
        case KeyEvent.KEYCODE_S: key_x = -1; break;
        case KeyEvent.KEYCODE_Q: key_z = -1; break;
        case KeyEvent.KEYCODE_E: key_z =  1; break;
        default:
            return super.onKeyDown(keyCode, msg);
        }
        controller.setPlayerDirection(Direction.fromKeys(
                key_x, key_y, key_z));        
        return true;
    }
    
    @Override
    public boolean onKeyUp(int keyCode, KeyEvent msg) {
        switch (keyCode) {
        case KeyEvent.KEYCODE_A:
        case KeyEvent.KEYCODE_D:
            key_y = 0;
            break;
        case KeyEvent.KEYCODE_W:
        case KeyEvent.KEYCODE_S:
            key_x = 0;
            break;
        case KeyEvent.KEYCODE_Q:
        case KeyEvent.KEYCODE_E:
            key_z = 0;
            break;
        case KeyEvent.KEYCODE_B:
            disconnect();
            break;
        default:
            return super.onKeyUp(keyCode, msg);
        }
        controller.setPlayerDirection(Direction.fromKeys(
                key_x, key_y, key_z));
        return true;
    }

    private void disconnect() {
        end();
        ((Netacka3D)(getContext())).initializeMenu();
    }

    /* (non-Javadoc)
     * @see android.view.View#onTouchEvent(android.view.MotionEvent)
     */
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        final float LEFT_X = getWidth()/2;
        final float RIGHT_X = getWidth()/2;
        float x = event.getX();
        //float y = event.getY();
        //Log.i("netacka", "MotionEvent X:"+x);
        
        switch (event.getAction()) {
        case MotionEvent.ACTION_DOWN:
        case MotionEvent.ACTION_MOVE:
            if (x <= LEFT_X) key_z = -1;
            else if(x >= RIGHT_X) key_z = 1;
            else key_z = 0;
            break;
        case MotionEvent.ACTION_UP:
            key_z = 0;
            renderer.spectateNext();
            break;
        default:
            return super.onTouchEvent(event);
        }
        /* STARA OBSŁUGA
        // granice podzialu ekranu na 9 obszarów
        // X oznacza wartosc wspolrzednej x
        final float LEFT_X = 2 * getWidth()/5 ;
        final float RIGHT_X = 4 * getWidth()/5;
        final float UP_Y = 2 * getHeight() / 5;
        final float DOWN_Y = 4 * getHeight() / 5;
        float x = event.getX();
        float y = event.getY();
        //Log.d("netacka", String.format("MotionEvent X:%f Y:%f", x, y));

        switch (event.getAction()) {
        case MotionEvent.ACTION_DOWN:
        case MotionEvent.ACTION_MOVE:
            
            if      (x<=LEFT_X)  key_y = 1;
            else if (x>=RIGHT_X) key_y = -1;
            else                 key_y = 0;
            
            if      (y<=UP_Y)    key_x = -1;
            else if (y>=DOWN_Y)  key_x = 1;
            else                 key_x = 0;
            
            break;
        case MotionEvent.ACTION_UP:
            renderer.spectateNext();
            key_x = key_y = key_z = 0;
            break;
        default:
            return super.onTouchEvent(event);
        } */
        controller.setPlayerDirection(Direction.fromKeys(
                key_x, key_y, key_z));
        return true;
    }

    public void processAccelerometer(float x, float y, float z) {
        //Logger.i("przetwarzam akcel");
        // na razie roboczo zakładamy poziomą pozycję wyjściową
        float xy = (float) Math.sqrt(x * x + y * y);
        key_x = key_y = key_z = 0;
        // xy i z powinny być dodatnie - z jest ujemne, jeśli telefon jest odwrócony
        if (xy > 0.2 * z) { // przechylenie krytyczne HACK magic number
            if (x > 0) {
                if (y < -sqrt3 * x) {
                    key_x = 0; key_y = -1;
                } else if (y * sqrt3 < -x) {
                    key_x = -1; key_y = -1;
                } else if (y * sqrt3 < x) {
                    key_x = -1; key_y = 0;
                } else if (y < sqrt3 * x) {
                    key_x = -1; key_y = 1;
                } else {
                    key_x = 0; key_y = 1;
                }
            } else { // x <= 0
                if (y > -sqrt3 * x) {
                    key_x = 0; key_y = 1;
                } else if (y * sqrt3 > -x) {
                    key_x = 1; key_y = 1;
                } else if (y * sqrt3 > x) {
                    key_x = 1; key_y = 0;
                } else if (y > sqrt3 * x) {
                    key_x = 1; key_y = -1;
                } else {
                    key_x = 0; key_y = -1;
                }                
            }
        }
        controller.setPlayerDirection(Direction.fromKeys(
                key_x, key_y, key_z));
    }

    public void end() {
        controller.disconnect();
        renderThread.interrupt();
        // renderer destroy (skąd wziąć GL?)
    } 
    
    class RenderThread extends Thread {
        private static final int DELAY = 1000/FPS;
        
        public void run() {
            try {
                while(true) {
                    sleep(DELAY);
                    requestRender();
                }
            } catch (InterruptedException e) {
                // nic
            }
        }
        
    }

}