/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.data.arena;

import java.io.IOException;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.StringTokenizer;

import javax.vecmath.*;

import netacka3d.misc.NumberParser;

public abstract class Importer {
    
    protected static float readFloat(StringTokenizer st) 
                                                throws DataFormatException {
        return NumberParser.parseFloat(st.nextToken());
    }
    
    protected static int readInt(StringTokenizer st) 
                                                throws DataFormatException {
        return NumberParser.parseInt(st.nextToken());
    }
    
    protected static Tuple3f readColor(StringTokenizer st) 
                                                throws DataFormatException {
        return new Vector3f(readFloat(st),
                            readFloat(st),
                            readFloat(st));
    }
    
    public static class DataFormatException extends IOException {
        private static final long serialVersionUID = 1L;
        public DataFormatException(Throwable cause) {
            super(cause);
        }
        public DataFormatException(String msg) {
            super(msg);
        }
    }
    
}
