/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.data.arena;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;

import javax.vecmath.Vector3f;

import netacka3d.data.arena.Mesh.Material;

public class MeshData implements Serializable {
    
    private static final long serialVersionUID = -14532145L;
    
    public Buffers[] bufs;
    public float[] bounds;
    public float[] startingPoints;

    public static class Buffers implements Serializable {
        private static final long serialVersionUID = 536765745624L;
        public float[] vs; // vertices
        public float[] ns; // normals
        public float[] tcs; // texture coordinates
        public Material material;
    }
    
    public Vector3f getOrigin() {
        assert bounds != null;
        return new Vector3f(bounds[0], bounds[1], bounds[2]);
    }
    
    public Vector3f getDelta() {
        return new Vector3f(bounds[3] - bounds[0], 
                            bounds[4] - bounds[1],
                            bounds[5] - bounds[2]);
    }
    
    public static MeshData loadFrom(InputStream input) 
                throws IOException {
        ObjectInputStream objInput = new ObjectInputStream(input);
        try {
            MeshData mD = (MeshData) objInput.readObject();
            assert mD.bufs != null;
            assert mD.startingPoints != null;
            assert mD.bounds != null;
            return mD;
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
    }
    
    public void writeTo(OutputStream output) 
                throws IOException, ClassNotFoundException {
        ObjectOutputStream objOutput = new ObjectOutputStream(output);
        objOutput.writeObject(this);
    }

}
