/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.logic;

import netacka3d.data.Segment;

public interface GameListener {
    
    /**
     * Obsługa zdarzenia: kontroler zaczął działanie (RUNNING)
     * @param playerId taki jest nasz playerId
     */
    void onStart(int playerId);
    
    /**
     * Obsługa zdarzenia: kontroler zakończył działanie (STOPPED) 
     */
    void onStop(String reason);
    
    /**
     * Obsługa zdarzenia: pojawił się nowy segment, lub aktualizowano 
     * istniejący, segment danego gracza dla danego kroku.
     * @param playerId gracz, o którego segment chodzi
     */
    void onSegment(int playerId, int t, Segment segment);
    
    /**
     * Obsługa zdarzenia: właśnie zaczął się krok game.stepNum czasu 
     * fizycznego. Jeśli stateChanged, to zmienił się stan gry: 
     * matchSettings, playersTable albo matchState.
     * @param stateChanged czy stan się zmienił w ciągu ostatniego kroku?
     */
    void onStep(boolean stateChanged, Game game);
    
}
