/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.logic.bots;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Random;
import java.util.Map.Entry;

import netacka3d.data.ArenaData;
import netacka3d.data.Direction;
import netacka3d.data.MatchSettings;
import netacka3d.data.Segment;
import netacka3d.logic.Game.MatchState;
import netacka3d.misc.Logger;

public class Sigma extends Bot {

    private MatchSettings myMS;
    private MatchSettings zeroMS;
    
    private ArrayList<Tentacle> tentacles;
    
    private Direction lastTurn;
    
    private static class Tentacle {
        Direction dir;
        float length;
        boolean collision;
        public Tentacle(Direction d, float l) {
            dir = d;
            length = l;
        }
    }
    
    public Sigma() {
        super();
        collisionsNeeded = true;
        Random r = new Random();
        tentacles = new ArrayList<Tentacle>();
        tentacles.add(new Tentacle(Direction.STRAIGHT, 10+r.nextFloat()));
        tentacles.add(new Tentacle(Direction.fromKeys(0, -1, 0), 10+r.nextFloat()));
        tentacles.add(new Tentacle(Direction.fromKeys(0, 1, 0), 10+r.nextFloat()));
        tentacles.add(new Tentacle(Direction.fromKeys(1, 0, 0), 10+r.nextFloat()));
        tentacles.add(new Tentacle(Direction.fromKeys(-1, 0, 0), 10+r.nextFloat()));
    }

    protected Direction doStep(boolean stateChanged) {
        if (game.matchState != MatchState.ROUND)
            return Direction.STRAIGHT;
        if (stateChanged) {
            myMS = game.matchSettings.getCopy();
            myMS.collide = true;
            zeroMS = game.matchSettings.getCopy();
            zeroMS.angleX *= 3;
            zeroMS.angleY *= 3;
            zeroMS.angleZ *= 3;
        }
        if (myMS == null) {
            Logger.w("null");
            return Direction.STRAIGHT;
        }
        Segment last = null;
        final ArenaData aD = game.arenaData;
        if (game.stepNum > 0)
            last = aD.getSegment(myPlayerId, game.stepNum-1);
        if (last != null) {
            
            Direction best = Direction.STRAIGHT;
            float bestLen = 0;
            
            for (Tentacle ten : tentacles) {
                Segment seg = calcSegment(last, ten.dir, ten.length);
                ten.collision = (aD.checkCollision(seg, myPlayerId) >= 0);
                ten.length *= ten.collision ? 0.8 : 1.2;
                if (ten.length > bestLen && !ten.dir.opposite().equals(lastTurn)) {
                    bestLen = ten.length;
                    best = ten.dir;
                }
            }
            lastTurn = best.equals(Direction.STRAIGHT) ? null : best;
            return best;
        }
        return Direction.STRAIGHT;
    }
    
    private Segment calcSegment(Segment base, Direction dir, float length) {
        Segment s = base.calcNext(dir, true, zeroMS);
        float oldLength = myMS.segmentLength;
        myMS.segmentLength = length;
        s = s.calcNext(Direction.STRAIGHT, true, myMS);
        myMS.segmentLength = oldLength;
        return s;
    }
}

