/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.misc;

import netacka3d.data.arena.Importer.DataFormatException;

public abstract class NumberParser {
    
    public static int parseInt(final String s) {
        return parseNumber(s, true);
    }
    
    /** uwaga: dla dużych floatów to nie działa */
    public static float parseFloat(final String s) {
        float f = parseNumber(s, false);
        int dot = s.indexOf('.');
        int len = s.length();
        int factor = 1;
        while (dot < len-1) {
            dot++;
            factor *= 10;
        }
        return f/factor;
    }
    
    // http://nadeausoftware.com/articles/2009/08/java_tip_how_parse_integers_quickly
    private static int parseNumber(final String s, boolean isInt) {
        
        if (s == null)
            throw new NumberFormatException("Null string");

        // Check for a sign.
        int sign = -1; // -1 oznacza "plus". Jest tak dziwnie, bo -MIN_VALUE > MAX_VALUE
        final int len = s.length();
        final char ch = s.charAt(0);
        int i = 0;
        if (ch == '-') {
            if (len == 1)
                throw new NumberFormatException("Missing digits: " + s);
            sign = 1;
            i = 1;
        }
        
        // Build the number.
        int num = 0;
        final int max = (sign == -1) ? -Integer.MAX_VALUE : Integer.MIN_VALUE;
        final int multmax = max / 10;
        while (i < len) {
            int d = s.charAt(i++);
            if (d == '.' && !isInt) 
                continue;
            d -= '0';
            if (d < 0 || d > 9)
                throw new NumberFormatException("Malformed: " + s);
            if (num < multmax)
                throw new NumberFormatException("Over/underflow: " + s + multmax);
            num *= 10;
            if (num < max+d)
                throw new NumberFormatException("Over/underflow: " + s);
            num -= d;
        }

        return sign * num;
    }
    
}
