/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d;

import javax.management.RuntimeErrorException;

import junit.framework.TestCase;

import org.junit.*;
import org.junit.runner.RunWith;
import org.junit.runners.Suite;
import org.junit.runners.Suite.SuiteClasses;

import static org.junit.Assert.*;

import netacka3d.data.PlayerColor;
import netacka3d.data.MatchSettings;
import netacka3d.logic.ClientController;
import netacka3d.logic.ServerController;
import netacka3d.logic.bots.RandomBot;
import netacka3d.logic.bots.SpinBot;
import netacka3d.misc.Logger;
import netacka3d.misc.Logger.Severity;
import netacka3d.network.ServerFinderTest;

@RunWith(Suite.class)
@SuiteClasses({
    ControllerTest.Test1.class,
    ControllerTest.Test2.class,
    ControllerTest.Test3.class,
    ControllerTest.Test4.class,
    ControllerTest.Test5.class,
})
public class ControllerTest {

    private static ClientController cc1, cc2, cc3;
    private static ServerController sc;
    private static int numThreads; 
    
    // ten cały cyrk jest po to, żeby JUnit tego nie odpalał równolegle
    public static class Test1 { @Test public void bla() { init(); i1(); sleep(500); s1(); sleep(500); f(); } }
    public static class Test2 { @Test public void bla() { init(); i2(); sleep(500); s2(); sleep(500); f(); } }
    public static class Test3 { @Test public void bla() { init(); i3(); sleep(500); s3(); sleep(500); f(); } }
    public static class Test4 { @Test public void bla() { init(); i4(); sleep(500); s4(); sleep(500); f(); } }
    public static class Test5 { @Test public void bla() { init(); i5(); sleep(500); s2(); sleep(500); f(); } }

    
    public static void init() {
        Init.init();
        cc1 = new ClientController();
        cc2 = new RandomBot();
        cc3 = new SpinBot();
        sc = new ServerController();
        numThreads = Thread.activeCount();
        Logger.i("");
        Logger.i("-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-");
    }
    
    
    public static void i1() {
        sc.startGame("test", new MatchSettings() {{
            matchWarmupTime = 0;
            matchFinishTime = 0;
            roundFinishTime = 0;
            roundWarmupTime = 0;
            arenaName = "cube";
        }}, 7357);
    }
    public static void i2() { // duża arena
        sc.startGame("test", new MatchSettings() {{
            roundFinishTime = 0;
            roundWarmupTime = 0;
            arenaName = "texturedCube";
            arenaSize = 1000;
        }}, 7357);
    }
    public static void i3() { // scoreLimit=1
        sc.startGame("test", new MatchSettings() {{
            matchWarmupTime = 0;
            matchFinishTime = 0;
            roundFinishTime = 0;
            roundWarmupTime = 0;
            arenaName = "cube";
            scoreLimit = 1;
        }}, 7357);
    }
    public static void i4() { // duże kąty
        sc.startGame("test", new MatchSettings() {{
            matchWarmupTime = 0;
            matchFinishTime = 0;
            roundFinishTime = 0;
            roundWarmupTime = 0;
            arenaName = "icosi";
            angleX = angleY = angleZ = 100;
        }}, 7357);
    }
    public static void i5() { // szeroki segment
        sc.startGame("test", new MatchSettings() {{
            arenaName = "cube";
            segmentWidth = 5;
        }}, 7357);
    }
    
    public static void s1() { // normalnie
        cc1.connect("s1.a", PlayerColor.RED,   "127.0.0.1", 7357);
        cc2.connect("s1.aa", PlayerColor.GREEN, "127.0.0.1", 7357);
        cc3.connect("s1.aaa", PlayerColor.BLUE,  "127.0.0.1", 7357);
        sleep(100);
        cc1.disconnect();
        cc2.disconnect();
        cc3.disconnect();
    }

    public static void s2() { // nie wszyscy sie rozlaczaja
        cc1.connect("s2.a", PlayerColor.RED,   "127.0.0.1", 7357);
        cc2.connect("s2.aa", PlayerColor.GREEN, "127.0.0.1", 7357);
        cc1.disconnect();
        cc3.connect("s2.aaa", PlayerColor.BLUE,  "127.0.0.1", 7357);
        sleep(100);
    }
    
    public static void s3() { // dużo połączeń, powtarzający się nick i kolor
        for (int i=0; i<10; i++) {
            ClientController cc = new ClientController();
            cc.connect("s3.x", PlayerColor.BLUE,  "127.0.0.1", 7357);
            sleep(100);
            cc.disconnect();
        }
    }
    public static void s4 () { // dużo graczy jednocześnie, kończą się kolory
        for (int i=0; i<10; i++) {
            ClientController cc = new ClientController();
            cc.connect("s4.x", PlayerColor.BLUE,  "127.0.0.1", 7357);
        }
        //nikt się nie rozłącza!
    }
    
    public static void f() {
        sc.endGame();
        Logger.i("-=-=-=-=- kończenie...");
        for (int i = 0; i < 50; i++) {
            sleep(100);
            if (Thread.activeCount() == numThreads)
                break;
        }
    }
    
    public static void sleep(int milis) {
        try {
            Thread.sleep(milis);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }
    
}
