/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;

import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import netacka3d.data.arena.Mesh;
import netacka3d.data.arena.MeshData;
import netacka3d.data.arena.ObjImporter;
import netacka3d.data.arena.Importer.DataFormatException;
import netacka3d.misc.Logger;

import org.junit.Before;
import org.junit.Test;


public class ObjImporterTest {

    private final static float EPSILON = 0.00001f;
    
    FileInputStream in = null;
    BufferedReader reader;
    
    @Before
    public void init() {
        Init.init();
    }
    
    @Test
    public void vertexTest() throws IOException, DataFormatException {
        Mesh m = ObjImporter.loadFromFile("cube.mesh");
        int fCount = 0;
        for (Mesh.Tri t : m.trisByMat.get(0)) {
            for (int i=0; i<3; i++) {
                Point3f expected = new Point3f(expectedV[fCount][i]);
                Point3f actual = t.vertices[i].point;
                assertTrue(actual.epsilonEquals(expected, EPSILON));
            }
            fCount++;
        }
        assertEquals(fCount, 2*6);
    }
    
    private float[][][] expectedV = {{{0,0,0}, {1,1,0}, {1,0,0}},
                                     {{0,0,0}, {0,1,0}, {1,1,0}},
                                     {{0,0,0}, {0,1,1}, {0,1,0}},
                                     {{0,0,0}, {0,0,1}, {0,1,1}},
                                     {{0,1,0}, {1,1,1}, {1,1,0}},
                                     {{0,1,0}, {0,1,1}, {1,1,1}},
                                     {{1,0,0}, {1,1,0}, {1,1,1}},
                                     {{1,0,0}, {1,1,1}, {1,0,1}},
                                     {{0,0,0}, {1,0,0}, {1,0,1}},
                                     {{0,0,0}, {1,0,1}, {0,0,1}},
                                     {{0,0,1}, {1,0,1}, {1,1,1}},
                                     {{0,0,1}, {1,1,1}, {0,1,1}}};

    @Test
    public void textureTest() throws IOException, DataFormatException {
        // powinno nie byc wyjatkow, np. FileNotFoundException przy ladowaniu mtl
        Mesh m = ObjImporter.loadFromFile("texturedCube.mesh");
        assertEquals(2, m.mats.size()); // domyślny + wczytany
        assertEquals("bla.jpg", m.mats.get(1).diffuseTexture);
    }
    
    @Test
    public void materialTest() throws IOException, DataFormatException {
        // dość duży i skomplikowany mesh
        Mesh m = ObjImporter.loadFromFile("icosi.mesh");
        assertEquals(3, m.mats.size());
        MeshData mD = m.getData();
        assertEquals(3.79, mD.getDelta().length(), 0.01); 
        assertTrue(m.trisByMat.get(2).get(3).vertices[2].normal != null);
    }

    
}
