/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.graphics.pc; //#.pc
//#.android package netacka3d.graphics.android;

import java.io.IOException;
import java.nio.ByteBuffer;

import javax.media.opengl.*;            //#.pc
import javax.media.opengl.glu.GLU;      //#.pc

import netacka3d.misc.FileIO;

//#.android import javax.microedition.khronos.egl.EGLConfig;
//#.android import javax.microedition.khronos.opengles.GL10;

//#replace GL GL10
//#replace glTexParameteri glTexParameterx

public class Texture {
    public int num;
    public int w, h;
    
    public Texture(GL gl, String fileName) {
        ByteBuffer buf;
        try {
            int[] dims = new int[2]; 
            buf = FileIO.loadImage(fileName, dims);
            w = dims[0];
            h = dims[1];
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        
        int[] tmpTex = new int[1];
        
        gl.glPixelStorei(GL.GL_UNPACK_ALIGNMENT, 1);
        gl.glGenTextures(1, tmpTex, 0);
        num = tmpTex[0];
        gl.glBindTexture(GL.GL_TEXTURE_2D, num);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_REPEAT);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_REPEAT);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR );
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR );
        buf.position(0);
        gl.glTexImage2D(GL.GL_TEXTURE_2D, 
                0, 
                GL.GL_RGBA,
                w,
                h,
                0, 
                GL.GL_RGBA, 
                GL.GL_UNSIGNED_BYTE, 
                buf);
    }
    
    public void destroy(GL gl) {
        int[] tmpTex = new int[] { num };
        gl.glDeleteTextures(1, tmpTex, 0);
    }
}
