/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * MatchSettingsEditor.java
 *
 * Created on 2010-04-27, 19:16:52
 */

package netacka3d.ui.pc;

import netacka3d.data.MatchSettings;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.*;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.Field;

/**
 *
 * @author janek
 */
public class MatchSettingsEditor extends javax.swing.JPanel {

    private MatchSettings mS;

    /** Creates new form MatchSettingsEditor */
    public MatchSettingsEditor() {
        initComponents();
    }

    public MatchSettingsEditor(MatchSettings mS) {
        set(mS);
        //initComponents();
        initialize();
    }

    public void initialize() {
        try {
            this.setLayout(new GridBagLayout());
            GridBagConstraints c = new GridBagConstraints();
            int gridy = 0;
            c.anchor = GridBagConstraints.LINE_END;
            c.fill = GridBagConstraints.HORIZONTAL;
            c.weightx = 0.0;
            c.ipadx = 4;
            c.ipady = 2;
            c.insets = new Insets(3, 1, 3, 1);
            
            for (Field f: MatchSettings.getFields()) {
                MatchSettings.S an = f.getAnnotation(MatchSettings.S.class);
                if (an == null)
                    continue;
    
                JComponent[] row = new JComponent[3];
                String help = "";
                
                row[0] = new JLabel(an.name());
                
                if (f.getName().equals("arenaName")) {
                    JComboBox box = new JComboBox(MatchSettings.arenaNames);
                    row[1] = box;
                    box.setSelectedItem(mS.arenaName);
                    box.addActionListener(new ArenaNameListener(box));
                } else {
                    JTextField field = new JTextField(6);
                    row[1] = field;
                    field.setMinimumSize(field.getPreferredSize());
                    field.getDocument().addDocumentListener(
                            new FieldListener(field, f, an));
                    field.getDocument().addDocumentListener(
                            new FieldListener(field, f, an));
                    field.setText(f.get(mS).toString());
                    if (f.getType() == float.class)
                        help = an.min() + " ~ " + an.max();
                    else if (f.getType() == int.class)
                        help = (int)an.min() + " ~ " + (int)an.max();                    
                }
                row[2] = new JLabel(help);
                
                c.gridy = gridy++;
                for (int i = 0; i < row.length; ++i) {
                    c.gridx = i;
                    if (row[i] != null)
                        this.add(row[i], c);
                }
            }
        } catch(IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }
    
    public void set(MatchSettings mS) {
        this.mS = mS.getCopy();
    }

    public MatchSettings get() {
        mS.update();
        return mS.getCopy();
    }

    private class FieldListener implements DocumentListener {
        private Field f;
        private MatchSettings.S an;
        private JTextField field;

        public FieldListener(JTextField field, Field f, MatchSettings.S an) {
            this.field = field;
            this.f = f;
            this.an = an;
        }
        
        private void update() {
            try {
                String text = field.getText();
                boolean valid = false;
                if (f.getType() == float.class) {
                    try {
                        float v = Float.parseFloat(text);
                        valid = an.min() <= v && v <= an.max();
                        if (valid)
                            f.setFloat(mS, v);
                    } catch(NumberFormatException e) {
                        //nic
                    }
                } else if (f.getType() == int.class) {
                    try {
                        int v = Integer.parseInt(text);
                        valid = an.min() <= v && v <= an.max();
                        if (valid)
                            f.setInt(mS, v);
                    } catch(NumberFormatException e) {
                        //nic
                    }
                } else if (f.getType() == String.class) {
                    valid = !text.equals("");
                    if (valid)
                        f.set(mS, text);
                }
                
                if (valid)
                    field.setForeground(Color.BLACK);
                else
                    field.setForeground(Color.RED);
                
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
        }

        public void insertUpdate(DocumentEvent ev) {
            update();
        }


        public void removeUpdate(DocumentEvent ev) {
            update();
        }


        public void changedUpdate(DocumentEvent ev) {
            update();
        }
    }
    
    private class ArenaNameListener implements ActionListener {
        JComboBox box;
        
        public ArenaNameListener(JComboBox box) {
            this.box = box;
        }

        public void actionPerformed(ActionEvent e) {
            mS.arenaName = (String)box.getSelectedItem();
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel10 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        maxPlayersField = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        scoreLimitField = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        speedSlider = new javax.swing.JSlider();
        ribbonWidthSlider = new javax.swing.JSlider();
        jLabel2 = new javax.swing.JLabel();
        arenaSizeSlider = new javax.swing.JSlider();

        jLabel10.setText("Arena size:");

        jLabel6.setText("Match:");

        jLabel7.setText("score limit:");

        jLabel9.setText("points");

        jLabel8.setText("max players:");

        scoreLimitField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                scoreLimitFieldActionPerformed(evt);
            }
        });

        jLabel1.setText("Speed:");

        jLabel2.setText("Ribbon width:");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jLabel6)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel7)
                    .addComponent(jLabel8))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(scoreLimitField)
                    .addComponent(maxPlayersField, javax.swing.GroupLayout.PREFERRED_SIZE, 69, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel9))
            .addGroup(layout.createSequentialGroup()
                .addComponent(jLabel10)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 28, Short.MAX_VALUE)
                .addComponent(arenaSizeSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 52, Short.MAX_VALUE)
                .addComponent(speedSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(ribbonWidthSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(jLabel7)
                    .addComponent(scoreLimitField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel9))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(maxPlayersField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel8))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel10)
                    .addComponent(arenaSizeSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(speedSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel1))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(ribbonWidthSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2))
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void scoreLimitFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_scoreLimitFieldActionPerformed
        mS.scoreLimit = Integer.parseInt(scoreLimitField.getText());
    }//GEN-LAST:event_scoreLimitFieldActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JSlider arenaSizeSlider;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JTextField maxPlayersField;
    private javax.swing.JSlider ribbonWidthSlider;
    private javax.swing.JTextField scoreLimitField;
    private javax.swing.JSlider speedSlider;
    // End of variables declaration//GEN-END:variables
}
